#pragma once
#include "FBullCowGame.h"
#include<time.h>
#include<map>
#include<iostream>

//using to make Unreal Syntax
#define TMAP std::map
using int32 = int;
using TCHAR = char;

EGameWinStatus FBullCowGame::IsGameWon(FBullCowCount Bull_Count) const//checks if player has enoughs bulls in guess to win and returns status
{
	if (Bull_Count.Bulls == GetHiddenWordLength()) { return EGameWinStatus::Enough_Bulls;}
	else { return EGameWinStatus::Not_Enough_Bulls;}
}

FBullCowGame::FBullCowGame(){Reset();}//Constructor--see reset for initializations

int32 FBullCowGame::GetHiddenWordLength()const { return MyHiddenWord.length();}

bool FBullCowGame::IsIsogram(FString TheGuess) const//checks the guess for repeating characters
{
	if (TheGuess.length() <= 1) { return true; }//treat 0 and 1 letter words as isogram
	TMAP<char, bool> LettersSeen;//setup the map
	for (auto Letter : TheGuess)//for all letters of the word
	//loop through letters of a word
	{
		Letter = tolower(Letter);//Handle Mixed Case
		//check if letters are in the map
		if (LettersSeen[Letter]) { return false; }
		else { LettersSeen[Letter] = true; }
	}
	return true;
}

bool FBullCowGame::IsLowerCase(FString TheGuess) const//Checks the guess for CAPITALS
{
	for (auto Letter : TheGuess)
	{
		if (!islower(Letter)) { return false; }
	}
	return true;
}

int32 FBullCowGame::GetMaxTries()const //gets max tries based on word length
{
	TMAP <int32, int32> WordLengthToMaxTries{ {5, 9}, {6, 11}, {7, 13}, {8, 15} };
	return WordLengthToMaxTries[MyHiddenWord.length()];
}

int32 FBullCowGame::GetCurrentTry() const{return MyCurrentTry;}//display current try

FString FBullCowGame::GetHiddenWord() const{return MyHiddenWord;}//display current hidden word for game over

EGuessStatus FBullCowGame::CheckGuessValidity(FString TheGuess)const//Checks guess for valid input and returns the guess status
{
	if (!IsIsogram(TheGuess)){return EGuessStatus::Not_Isogram;}//check for repeating characters
	else if (TheGuess.length()!=GetHiddenWordLength()){return EGuessStatus::Wrong_Length;}//if not correct length
	else if (!IsLowerCase(TheGuess)) { return EGuessStatus::NOT_LOWERCASE; }
	else{return EGuessStatus::ok;}
}

TMAP <int32, FString> HiddenWordMap//Words for hidden guess--Can add more here
{
	//MUST BE ISOGRAMS ELSE GAME CANT BE PLAYED--see IsIsogram
	{ 1, "power" },{ 2,"stick" },{ 3,"comedy" },
	{ 4,"wizard" },{ 5,"magic" },{ 6,"house" },
	{ 7,"eight" },{ 8,"sharp" },{ 9,"sight" },
	{ 10,"knight" },{ 11,"solar" },{ 12,"vertical" },
	{ 13,"desktop" },{ 14,"terminal" },{ 15,"symbolic" },
	{ 16,"storage" },{ 17, "shark" },{ 18,"rodent" },
	{ 19,"flower" },{ 20,"cloud" },{ 21,"teach" },
	{ 22,"dialogue" },{ 23,"english" },{ 24,"phone" },
	{ 25,"table" },{ 26,"isogram" },{ 27,"triangle" },
	{ 28,"python" },{ 29,"single" },{ 30,"double" },
	{ 31,"graphics" },{ 32,"geomancy" }
};

void const FBullCowGame::Reset()//initializes private members
{
	MyCurrentTry = 1;
	srand((unsigned int) time(NULL));
	FString HIDDEN_WORD = HiddenWordMap[rand() % HiddenWordMap.size() + 1];//Gets a hidden word from the map
	MyHiddenWord = HIDDEN_WORD;
}

//counts bull and cows and increases try number assuming valid guess
FBullCowCount FBullCowGame::SubmitValidGuess(FString TheGuess)
{
	MyCurrentTry++;
	FBullCowCount BULL_COW_COUNT;
	int32 hWordLength = MyHiddenWord.length();
	for (int32 MyHiddenChar = 0; MyHiddenChar < hWordLength; MyHiddenChar++){//Loop through all letters in Hidden Word
		for (int32 MyGuessChar = 0; MyGuessChar <  (int32)TheGuess.length(); MyGuessChar++){//loop through all letters in players VALID guess
			if (MyHiddenWord[MyHiddenChar] == TheGuess[MyGuessChar])//Checks if indexes are equal
				if (MyHiddenChar == MyGuessChar)//checks if values in indexes are equal then increase bulls
					BULL_COW_COUNT.Bulls++;
				else if (MyGuessChar == hWordLength)break;//exits inner loop when guess char index equals the hidden words length.
				else{BULL_COW_COUNT.Cows++;}//if break not encountered, increase cows
			else if (MyHiddenChar == hWordLength)break;//exits outer loop when hidden char index equals the hidden words length.
		}
	}return BULL_COW_COUNT;
}
//test