/*This is the a console executable which makes use of the bull cow class
acts as the view in a MVC pattern and is responsible for user interaction
for game logic see the included file*/
#pragma once
#include<iostream>
#include"FBullCowGame.h"

//using for Unreal Friendly Syntax
using int32 = int;
using FText = std::string;

//Function Prototypes Outside Of Class
void PrintIntro();//introduces game
void PlayGame();//uses game logic
FText GetValidGuess();
bool AskToPlayAgain();
FBullCowGame BCGAME;//instantiation of the class--re used across plays
void GameOver(EGameWinStatus);//concludes game

int main()//Main-> Entry point for application
{
	//This is where the 'magic' happens
	do
	{
		BCGAME.Reset();
		PlayGame();
	} while (AskToPlayAgain()==true);//keep looping until player quits
	return 0;
}

void PrintIntro()//Function to introduce game
{
	int32 TheWordLength = BCGAME.GetHiddenWordLength();
	std::cout << "Welcome To Bulls & Cows, A simple text based word game.\n";
	std::cout << "          /\\ _/\\       <^~~~^>\n";
	std::cout << " `\\       \\ o o/        \\o o/     /`\n";
	std::cout << "   \\ ~~~~~~\\ ..\\         \\./~~~~~/\n";
	std::cout << "   | Bulls |\\--/    &     | Cows |\n";
	std::cout << "   | |---| |              | |--| |\n";
	std::cout << "   ^-^   ^-^              ^-^  ^-^\n" << std::endl;
	std::cout << "Can you guess the " << TheWordLength << " letter isogram?\n";
	std::cout << std::endl;
}

//player submits guess here
//when guess returns checks bull count
//to determine if game is won
void PlayGame()
{
	PrintIntro();//introduce game
	int32 MAX_TURNS = BCGAME.GetMaxTries();
	EGameWinStatus WinLoseConditions = EGameWinStatus::DEFAULT;
	do
	{	
		FText Guess = GetValidGuess();
		FBullCowCount BULLS_COWS = BCGAME.SubmitValidGuess(Guess);
		std::cout << "\nYou Got " << BULLS_COWS.Bulls << " Bulls and ";
		std::cout << BULLS_COWS.Cows << " Cows\n\n";
		WinLoseConditions = BCGAME.IsGameWon(BULLS_COWS);
		switch (WinLoseConditions)
		{
		case EGameWinStatus::Not_Enough_Bulls:
			std::cout << "Sorry, thats incorrect\n\n";
			break;
		case EGameWinStatus::DEFAULT:
			std::cout << "Something went wrong updating bull count\n";
			break;
		default:
			break;
		}
	} while ((WinLoseConditions != EGameWinStatus::Enough_Bulls) && (BCGAME.GetCurrentTry() <= MAX_TURNS));
	GameOver(WinLoseConditions);
}

FText GetValidGuess()
{
	//submit valid guess to game
	//get a guess from the player
	FText Guess = "";
	EGuessStatus CheckValidGuess = EGuessStatus::INVALID_STATUS;
	do {
		int32 MaxTrys = BCGAME.GetMaxTries();
		int32 CurrentTry = BCGAME.GetCurrentTry();
		std::cout << "Try: " << CurrentTry << " of " << MaxTrys << std::endl;
		std::cout << "Enter your guess: ";
		std::getline(std::cin, Guess);
		//Check for valid guess
		CheckValidGuess = BCGAME.CheckGuessValidity(Guess);
		switch (CheckValidGuess)
		{
		case EGuessStatus::INVALID_STATUS:
			std::cout << "Something Went Wrong Here\n";
			break;
		case EGuessStatus::Not_Isogram:
			std::cout << "Please enter a word with no repeating letters\n\n";
			break;
		case EGuessStatus::Wrong_Length:
			std::cout << "Please enter a " << BCGAME.GetHiddenWordLength() << " letter isogram\n\n";
			break;
		case EGuessStatus::NOT_LOWERCASE:
			std::cout << "Please enter your guess in lowercase\n\n";
		default:
			break;
		}
	} while (CheckValidGuess != EGuessStatus::ok);//exit if valid guess is ok
	return Guess;
}

//after game completes, ask to play again
//bool for True or False
bool AskToPlayAgain()
{
	std::cout << "Do you want to play again? ";
	FText Response = "";
	std::getline(std::cin, Response);
	std::cout << "\n";
	bool bPlayAgain = (Response[0] == 'y') || (Response[0] == 'Y');
	return bPlayAgain;
}

void GameOver(EGameWinStatus WinConditions)//wraps up game. Shows player word when tries exhausted
{
	if (WinConditions == EGameWinStatus::Not_Enough_Bulls)
	{
		std::cout << "The hidden word was: " << BCGAME.GetHiddenWord() << std::endl;
		std::cout << "\nGame Over!\n";
		std::cout << "Thanks for playing!!\n" << std::endl;
	}
	else
	{
		std::cout << "Good Job, Your Guess Was Correct\n" << std::endl;
	}
}
